import json
import os
import threading
from time import sleep

from game.utility import get_connected_users, STATE_FILEPATH, write_to_user_teminal


class Game:
    def __init__(self):
        self.players = dict()
        self.finished_players = dict()
        self.__is_running = False

    def __update_players(self):
        user_names = get_connected_users()
        for username in user_names:
            if username not in self.players:
                self.players[username] = Player(username)
                print("\nAdding player: {}".format(username))

    def __loop(self):
        while self.__is_running:
            # update connected users
            self.__update_players()
            for k, p in self.players.items():
                # create statefiles is not existing
                if not os.path.exists(p.state_file):
                    f = open(p.state_file, "w")
                    f.write(str(p.current_task))
                    f.close()
                # check solutions and set next task
                if not p.has_finished:
                    if p.is_current_task_done:
                        p.current_task_index += 1
                        # write new task to statefile
                        f = open(p.state_file, "w")
                        f.write(str(p.current_task))
                        f.close()
                        # print dialog to user
                        if p.current_task is not None:
                            if p.current_task.task_dialog != "":
                                write_to_user_teminal(k, p.current_task.task_dialog)
                else:
                    # if player is done
                    if k not in self.finished_players:
                        self.finished_players[k] = p
                        write_to_user_teminal(k, "Super gemacht! Du bist mit den Aufgaben durch! Wenn du magst kannst du noch mal den Zug mit sl fahren lassen.")

            # print state
            print(
                "\rCurrent players: {}\tdone: {}".format(
                    len(self.players), len(self.finished_players)
                ),
                end="",
            )
            sleep(0.1)

    def start(self):
        self.__is_running = True
        threading.Thread(target=self.__loop())

    def stop(self):
        self.__is_running = False


class GameTask:
    def __init__(self, solution_checker, task_text, task_help, task_dialog=""):
        self.solution_checker = solution_checker
        self.task_text = task_text
        self.task_help = task_help
        self.task_dialog = task_dialog

    def is_task_done(self, username):
        return self.solution_checker(username)

    def __str__(self):
        d = dict(text=self.task_text, help=self.task_help)
        return json.dumps(d)


class Player:
    def __init__(self, username):
        self.username = username
        self.current_task_index = 0
        from game.tasks import GAME_TASK_LIST

        self.__task_list = GAME_TASK_LIST
        self.state_file = "{}{}_state.json".format(STATE_FILEPATH, username)

    @property
    def current_task(self):
        return self.__task_list[self.current_task_index]

    @property
    def is_current_task_done(self) -> bool:
        return self.current_task.is_task_done(self.username)

    @property
    def has_finished(self):
        return self.current_task is None
